/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 * 
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 * 
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common Development
 * and Distribution License("CDDL") (collectively, the "License").  You
 * may not use this file except in compliance with the License. You can obtain
 * a copy of the License at https://glassfish.dev.java.net/public/CDDL+GPL.html
 * or glassfish/bootstrap/legal/LICENSE.txt.  See the License for the specific
 * language governing permissions and limitations under the License.
 * 
 * When distributing the software, include this License Header Notice in each
 * file and include the License file at glassfish/bootstrap/legal/LICENSE.txt.
 * Sun designates this particular file as subject to the "Classpath" exception
 * as provided by Sun in the GPL Version 2 section of the License file that
 * accompanied this code.  If applicable, add the following below the License
 * Header, with the fields enclosed by brackets [] replaced by your own
 * identifying information: "Portions Copyrighted [year]
 * [name of copyright owner]"
 * 
 * Contributor(s):
 * 
 * If you wish your version of this file to be governed by only the CDDL or
 * only the GPL Version 2, indicate your decision by adding "[Contributor]
 * elects to include this software in this distribution under the [CDDL or GPL
 * Version 2] license."  If you don't indicate a single choice of license, a
 * recipient has the option to distribute your version of this file under
 * either the CDDL, the GPL Version 2 or to extend the choice of license to
 * its licensees as provided above.  However, if you add GPL Version 2 code
 * and therefore, elected the GPL Version 2 license, then the option applies
 * only if the new code is made subject to such option by the copyright
 * holder.
 */

/*
 * Copyright 2004-2005 Sun Microsystems, Inc.  All rights reserved.
 * Use is subject to license terms.
 */

package com.sun.enterprise.admin.monitor.stats;

import javax.management.j2ee.statistics.RangeStatistic;
import javax.management.j2ee.statistics.CountStatistic;
import javax.management.j2ee.statistics.Stats;

/**
 * A Stats interface to represent the statistical data about 
 * Work Management in the Connector Module
 * 
 * @author  Murali Vempaty
 * @since   SJSAS8.1
 */

public interface ConnectorWorkMgmtStats extends Stats {
    
    /**
     * returns the current, low & high counts of the work objects executed for
     * a connector module since the last reset. This is an aggregate of all the
     * doWork, doSchedule, and doStart work objects initiated by the connector
     * module
     * @return RangeStatistic
     */
    public RangeStatistic getActiveWorkCount();
    
    /**
     * indicates the current, high & low of the number of work objects waiting 
     * in the work queue before executing, since the last reset
     * @return RangeStatistic
     */
    public RangeStatistic getWaitQueueLength();
    
    /**
     * indicates the longest and shorted wait of a work object in the work queue
     * before it gets executed, since the last reset
     * @return RangeStatistic
     */
    public RangeStatistic getWorkRequestWaitTime();
    
    /**
     * indicates the number of work objects submitted by a connector module 
     *  for execution, since the last reset
     * @return CountStatistic
     */
    public CountStatistic getSubmittedWorkCount();
    
    /**
     * indicates the number of work objects rejected by the Application Server
     * per connector module, since the last reset.
     * @return CountStatistic
     */
    public CountStatistic getRejectedWorkCount();
    
    /**
     * indicates the number of work objects that were completed by the 
     * Application Server per connector module, since the last reset.
     * @return CountStatistic
     */
    public CountStatistic getCompletedWorkCount();
    
}
