/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 * 
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 * 
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common Development
 * and Distribution License("CDDL") (collectively, the "License").  You
 * may not use this file except in compliance with the License. You can obtain
 * a copy of the License at https://glassfish.dev.java.net/public/CDDL+GPL.html
 * or glassfish/bootstrap/legal/LICENSE.txt.  See the License for the specific
 * language governing permissions and limitations under the License.
 * 
 * When distributing the software, include this License Header Notice in each
 * file and include the License file at glassfish/bootstrap/legal/LICENSE.txt.
 * Sun designates this particular file as subject to the "Classpath" exception
 * as provided by Sun in the GPL Version 2 section of the License file that
 * accompanied this code.  If applicable, add the following below the License
 * Header, with the fields enclosed by brackets [] replaced by your own
 * identifying information: "Portions Copyrighted [year]
 * [name of copyright owner]"
 * 
 * Contributor(s):
 * 
 * If you wish your version of this file to be governed by only the CDDL or
 * only the GPL Version 2, indicate your decision by adding "[Contributor]
 * elects to include this software in this distribution under the [CDDL or GPL
 * Version 2] license."  If you don't indicate a single choice of license, a
 * recipient has the option to distribute your version of this file under
 * either the CDDL, the GPL Version 2 or to extend the choice of license to
 * its licensees as provided above.  However, if you add GPL Version 2 code
 * and therefore, elected the GPL Version 2 license, then the option applies
 * only if the new code is made subject to such option by the copyright
 * holder.
 */
package com.sun.enterprise.management.config;

import java.util.Map;

import javax.management.ObjectName;

import com.sun.appserv.management.base.XTypes;

import com.sun.appserv.management.config.Libraries;

import com.sun.appserv.management.util.misc.StringUtil;
import com.sun.appserv.management.util.misc.ExceptionUtil;
import com.sun.appserv.management.util.stringifier.ArrayStringifier;

import com.sun.enterprise.management.support.Delegate;

/**
    Base class for certain subclasses 
 */
public class DeployedItemConfigBase extends AMXConfigImplBase
    implements Libraries, ConfigFactoryCallback
{
		public
	DeployedItemConfigBase( final Delegate delegate )
	{
		super( delegate );
	}
	

        protected static String
	getPathSeparator()
	{
	    return System.getProperty( "path.separator" );
	}
	
	private static final String LIBRARIES_ATTR  = "Libraries";
	
	    public String[]
	getLibraries()
	{
	    final String s  = (String)delegateGetAttributeNoThrow( LIBRARIES_ATTR );
	    
	    final String[]  libraries = s == null ?
	        EMPTY_STRING_ARRAY : s.split( getPathSeparator() );
	        
	    return libraries;
	}
	
	/**
	    Note: some system apps make this Attribute read-only,
	    though the MBeanInfo advertises it as read/write.
	 */
	    public void
	setLibraries( final String[] libraries )
	{
	    if ( libraries == null || libraries.length == 0 )
	    {
	        final String[]  current = getLibraries();
	        if ( current.length == 0 )
	        {
	            // OK, no change
	        }
	        else
	        {
                delegateSetAttributeNoThrow( LIBRARIES_ATTR, null );
            }
	    }
	    else
	    {
    	    final String SEP    = getPathSeparator();
    	    
    	    for( final String s : libraries )
    	    {
    	        if ( s == null || s.length() == 0 || s.indexOf( SEP ) >= 0 )
    	        {
    	            throw new IllegalArgumentException( StringUtil.quote( "" + s ) );
    	        }
    	    }
    	    final String libs   = StringUtil.toString( SEP, (Object[])libraries );
            
            delegateSetAttributeNoThrow( LIBRARIES_ATTR, libs );
        }
	}
}







