/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 * 
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 * 
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common Development
 * and Distribution License("CDDL") (collectively, the "License").  You
 * may not use this file except in compliance with the License. You can obtain
 * a copy of the License at https://glassfish.dev.java.net/public/CDDL+GPL.html
 * or glassfish/bootstrap/legal/LICENSE.txt.  See the License for the specific
 * language governing permissions and limitations under the License.
 * 
 * When distributing the software, include this License Header Notice in each
 * file and include the License file at glassfish/bootstrap/legal/LICENSE.txt.
 * Sun designates this particular file as subject to the "Classpath" exception
 * as provided by Sun in the GPL Version 2 section of the License file that
 * accompanied this code.  If applicable, add the following below the License
 * Header, with the fields enclosed by brackets [] replaced by your own
 * identifying information: "Portions Copyrighted [year]
 * [name of copyright owner]"
 * 
 * Contributor(s):
 * 
 * If you wish your version of this file to be governed by only the CDDL or
 * only the GPL Version 2, indicate your decision by adding "[Contributor]
 * elects to include this software in this distribution under the [CDDL or GPL
 * Version 2] license."  If you don't indicate a single choice of license, a
 * recipient has the option to distribute your version of this file under
 * either the CDDL, the GPL Version 2 or to extend the choice of license to
 * its licensees as provided above.  However, if you add GPL Version 2 code
 * and therefore, elected the GPL Version 2 license, then the option applies
 * only if the new code is made subject to such option by the copyright
 * holder.
 */
package com.sun.enterprise.admin.wsmgmt.stats.impl;

import javax.management.j2ee.statistics.Stats;
import javax.management.j2ee.statistics.Statistic;
import javax.management.j2ee.statistics.CountStatistic;
import javax.management.j2ee.statistics.BoundedRangeStatistic;

import com.sun.enterprise.admin.monitor.registry.MonitoringRegistry;
import com.sun.enterprise.admin.monitor.registry.MonitoringLevelListener;
import com.sun.enterprise.admin.monitor.registry.MonitoringLevel;
import com.sun.enterprise.admin.monitor.registry.MonitoringRegistrationException;

import com.sun.enterprise.server.ApplicationServer;

import com.sun.enterprise.admin.monitor.stats.GenericStatsImpl;

import java.util.logging.Logger;
import com.sun.enterprise.log.Log;
import com.sun.logging.LogDomains;

/**
 * Base class for all the StatsImpl. Also provides a couple
 *  of methods for MonitorListener
 *
 * @author Satish Viswanatham
 */
public abstract class StatsImpl
    implements Stats
{
    protected final static Logger _logger =
        LogDomains.getLogger(LogDomains.EJB_LOGGER);

    private GenericStatsImpl	    genericStatsDelegate;
    protected boolean		    monitorOn = false;
    protected boolean		    registered = true;
    private MonitoringLevel	    currentMonitoringLevel;

    protected StatsImpl() {
    }
    
    protected void initialize(String statInterfaceName) {
	try {
	    genericStatsDelegate =  new GenericStatsImpl(statInterfaceName, this);
	} catch(ClassNotFoundException cnfEx) {
	    throw new RuntimeException(statInterfaceName + " not found", cnfEx);
	}
    }

    public Statistic getStatistic(String statName) {
	return genericStatsDelegate.getStatistic(statName);
    }

    public String[] getStatisticNames() {
	return genericStatsDelegate.getStatisticNames();
    }

    public Statistic[] getStatistics() {
	return genericStatsDelegate.getStatistics();
    }

    public String statToString() {
	StringBuffer sbuf = new StringBuffer();
	Statistic[] stats = getStatistics();
	int sz = stats.length;
	for (int i=0; i<sz; i++) {
	    if (stats[i] instanceof CountStatistic) {
		CountStatistic stat = (CountStatistic) stats[i];
		sbuf.append(stat.getName()).append("=")
		    .append(stat.getCount()).append("; ");
	    } else if (stats[i]  instanceof BoundedRangeStatistic) {
		BoundedRangeStatistic stat = (BoundedRangeStatistic) stats[i];
		sbuf.append(stat.getName()).append("=")
		    .append(stat.getCurrent()).append("; ");
	    } else {
		sbuf.append(stats[i].getName()).append("=?");
	    }
	}

	return sbuf.toString();
    }
}
