/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 * 
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 * 
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common Development
 * and Distribution License("CDDL") (collectively, the "License").  You
 * may not use this file except in compliance with the License. You can obtain
 * a copy of the License at https://glassfish.dev.java.net/public/CDDL+GPL.html
 * or glassfish/bootstrap/legal/LICENSE.txt.  See the License for the specific
 * language governing permissions and limitations under the License.
 * 
 * When distributing the software, include this License Header Notice in each
 * file and include the License file at glassfish/bootstrap/legal/LICENSE.txt.
 * Sun designates this particular file as subject to the "Classpath" exception
 * as provided by Sun in the GPL Version 2 section of the License file that
 * accompanied this code.  If applicable, add the following below the License
 * Header, with the fields enclosed by brackets [] replaced by your own
 * identifying information: "Portions Copyrighted [year]
 * [name of copyright owner]"
 * 
 * Contributor(s):
 * 
 * If you wish your version of this file to be governed by only the CDDL or
 * only the GPL Version 2, indicate your decision by adding "[Contributor]
 * elects to include this software in this distribution under the [CDDL or GPL
 * Version 2] license."  If you don't indicate a single choice of license, a
 * recipient has the option to distribute your version of this file under
 * either the CDDL, the GPL Version 2 or to extend the choice of license to
 * its licensees as provided above.  However, if you add GPL Version 2 code
 * and therefore, elected the GPL Version 2 license, then the option applies
 * only if the new code is made subject to such option by the copyright
 * holder.
 */

package com.sun.enterprise.deployment.annotation;

import java.util.Map;
import java.lang.annotation.Annotation;

/**
 * This interface defines the result of an annotation processing 
 * returned by an annotation handler. Annotation handlers are supposed
 * to handle ONE annotation at a time. However, this simplistic 
 * approach does not always work as annotations on a element can have 
 * interdependencies. 
 *
 * This could be resolved trough sophisticated caching in the context
 * for instance but it is usually easier to notify the processor that 
 * a particular annotation handler has processed more than one annotation 
 * (which shouldn't be reprocessed subsequently...)
 *
 * @author Jerome Dochez
 */
public interface HandlerProcessingResult {
    
    /** 
     * Returns a map of annotation types processed by this handler as keys 
     * and a ResultType as a value to communicate the result of the annotation 
     * processing for each processed annotation types.
     * @return the map of processed annoation types and result.
     */
    public Map<Class<? extends Annotation>,ResultType> processedAnnotations();    
    
    /**
     * @return the overall result for this handler processing. Since handlers 
     * can process more than one annotation, this overall result will contain
     * the most severe error the haandler encountered while processing the
     * annotations.
     */
     public ResultType getOverallResult();
}
