/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 * 
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 * 
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common Development
 * and Distribution License("CDDL") (collectively, the "License").  You
 * may not use this file except in compliance with the License. You can obtain
 * a copy of the License at https://glassfish.dev.java.net/public/CDDL+GPL.html
 * or glassfish/bootstrap/legal/LICENSE.txt.  See the License for the specific
 * language governing permissions and limitations under the License.
 * 
 * When distributing the software, include this License Header Notice in each
 * file and include the License file at glassfish/bootstrap/legal/LICENSE.txt.
 * Sun designates this particular file as subject to the "Classpath" exception
 * as provided by Sun in the GPL Version 2 section of the License file that
 * accompanied this code.  If applicable, add the following below the License
 * Header, with the fields enclosed by brackets [] replaced by your own
 * identifying information: "Portions Copyrighted [year]
 * [name of copyright owner]"
 * 
 * Contributor(s):
 * 
 * If you wish your version of this file to be governed by only the CDDL or
 * only the GPL Version 2, indicate your decision by adding "[Contributor]
 * elects to include this software in this distribution under the [CDDL or GPL
 * Version 2] license."  If you don't indicate a single choice of license, a
 * recipient has the option to distribute your version of this file under
 * either the CDDL, the GPL Version 2 or to extend the choice of license to
 * its licensees as provided above.  However, if you add GPL Version 2 code
 * and therefore, elected the GPL Version 2 license, then the option applies
 * only if the new code is made subject to such option by the copyright
 * holder.
 */

/**
 * PROPRIETARY/CONFIDENTIAL.  Use of this product is subject to license terms.
 *
 * Copyright 2001-2002 by iPlanet/Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 */
package com.sun.enterprise.config.impl;

import java.io.Serializable;
import java.util.HashMap;
import java.util.Set;
import java.util.Iterator;

/**
 * A configuration change for an element. Holds xpath, list of changed 
 * attributes, their old and new values.
 */
public class ConfigUpdateImpl extends ConfigChangeImpl implements com.sun.enterprise.config.ConfigUpdate, Serializable {

   // private String xpath;
    private HashMap oldValues = new HashMap();
    private HashMap newValues = new HashMap();
    
    public ConfigUpdateImpl(String xpath, String attrName, String oldValue, String newValue) {
        this.xpath = xpath;
        addChangedAttribute(attrName, oldValue, newValue);
    }
    
    public String getConfigChangeType() {
        return TYPE_UPDATE;
    }
        
    /**
     * add attr changes to the hashmap
     */
    public void addChangedAttribute(String attrName, String oldValue, String newValue) {
        //FIXME: This solution will not work for cases involving setAttribute(A, NULL)
        // null has to be handled gracefully in future releases.
        
        if(oldValues.get(attrName) == null) {
           // if(!oldValue.equals(newValue)) {
                oldValues.put(attrName, oldValue);
                newValues.put(attrName, newValue);
           // } else {
           //     return;
           // }   
        } else if(oldValues.get(attrName).equals(newValue)) {
            oldValues.remove(attrName);
            newValues.remove(attrName);
        } else {
            newValues.put(attrName, newValue);
        }
    }
    
    public String getOldValue(String attrName) {
        return (String)oldValues.get(attrName);
    }
    
    public String getNewValue(String attrName) {
        return (String)newValues.get(attrName);
    }
    
    public Set getAttributeSet() {
        return oldValues.keySet();
    }

    public void removeAttribute(String attrName) {
        if (attrName == null) {
            throw new IllegalArgumentException("Attribute name is null");
        }
        if (oldValues.containsKey(attrName) && newValues.containsKey(attrName)) {
            oldValues.remove(attrName);
            newValues.remove(attrName);
        }
    }

    public String toString() {
        String ret = "update xpath=" +xpath +"\n";
        Set s = newValues.keySet();
        for(Iterator iter = s.iterator(); iter.hasNext();) {
            String next = (String) iter.next();
            ret+= "\t" + "attr=" + next + "   " + oldValues.get(next) + "-->" + newValues.get(next) + "\n";
        }
        return ret;
    }
    
    public String getName() {
        return null; //FIXME TBD
    }
}