/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 * 
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 * 
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common Development
 * and Distribution License("CDDL") (collectively, the "License").  You
 * may not use this file except in compliance with the License. You can obtain
 * a copy of the License at https://glassfish.dev.java.net/public/CDDL+GPL.html
 * or glassfish/bootstrap/legal/LICENSE.txt.  See the License for the specific
 * language governing permissions and limitations under the License.
 * 
 * When distributing the software, include this License Header Notice in each
 * file and include the License file at glassfish/bootstrap/legal/LICENSE.txt.
 * Sun designates this particular file as subject to the "Classpath" exception
 * as provided by Sun in the GPL Version 2 section of the License file that
 * accompanied this code.  If applicable, add the following below the License
 * Header, with the fields enclosed by brackets [] replaced by your own
 * identifying information: "Portions Copyrighted [year]
 * [name of copyright owner]"
 * 
 * Contributor(s):
 * 
 * If you wish your version of this file to be governed by only the CDDL or
 * only the GPL Version 2, indicate your decision by adding "[Contributor]
 * elects to include this software in this distribution under the [CDDL or GPL
 * Version 2] license."  If you don't indicate a single choice of license, a
 * recipient has the option to distribute your version of this file under
 * either the CDDL, the GPL Version 2 or to extend the choice of license to
 * its licensees as provided above.  However, if you add GPL Version 2 code
 * and therefore, elected the GPL Version 2 license, then the option applies
 * only if the new code is made subject to such option by the copyright
 * holder.
 */

package com.sun.ejb.containers.interceptors;

import java.lang.reflect.Method;
import javax.interceptor.InvocationContext;
import javax.ejb.EJBContext;

import com.sun.ejb.EJBUtils;
import com.sun.ejb.Invocation.InterceptorChain;
import com.sun.ejb.containers.EJBContextImpl;

import java.util.Map;
import java.util.HashMap;
import java.util.logging.Level;
import java.util.logging.Logger;


/**
 * Concrete InvocationContext implementation passed to callback methods 
 * defined in interceptor classes.
 */
public class CallbackInvocationContext implements InvocationContext {

    private EJBContextImpl ejbContext;
    private Map contextData;
    private int callbackIndex = 0;
    private CallbackChainImpl callbackChain;

    public CallbackInvocationContext(EJBContextImpl ejbContextImpl,
                                     CallbackChainImpl chain) {
        ejbContext = ejbContextImpl;
        callbackChain = chain;
    }

    public Object getTarget() {
        return ejbContext.getEJB();
    }

    public Method getMethod() {
        return null;
    }
    
    public Object[] getParameters() {
        throw new IllegalStateException("not applicable to Callback methods");
    }

    public void setParameters(Object[] params) {
        throw new IllegalStateException("not applicable to Callback methods");
    }

    public EJBContext getEJBContext() {
        return ejbContext;
    }

    public Map<String, Object> getContextData() {
        if( contextData == null ) {
            contextData = new HashMap();
        }

        return contextData;
    }
    
    public Object proceed() throws Exception {
        try {
            callbackIndex++;
            return callbackChain.invokeNext(callbackIndex, this);
        } catch (Exception ex) {
            throw ex;
        } catch (Throwable th) {
            throw new Exception(th);
        }
    }

}

