/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 * 
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 * 
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common Development
 * and Distribution License("CDDL") (collectively, the "License").  You
 * may not use this file except in compliance with the License. You can obtain
 * a copy of the License at https://glassfish.dev.java.net/public/CDDL+GPL.html
 * or glassfish/bootstrap/legal/LICENSE.txt.  See the License for the specific
 * language governing permissions and limitations under the License.
 * 
 * When distributing the software, include this License Header Notice in each
 * file and include the License file at glassfish/bootstrap/legal/LICENSE.txt.
 * Sun designates this particular file as subject to the "Classpath" exception
 * as provided by Sun in the GPL Version 2 section of the License file that
 * accompanied this code.  If applicable, add the following below the License
 * Header, with the fields enclosed by brackets [] replaced by your own
 * identifying information: "Portions Copyrighted [year]
 * [name of copyright owner]"
 * 
 * Contributor(s):
 * 
 * If you wish your version of this file to be governed by only the CDDL or
 * only the GPL Version 2, indicate your decision by adding "[Contributor]
 * elects to include this software in this distribution under the [CDDL or GPL
 * Version 2] license."  If you don't indicate a single choice of license, a
 * recipient has the option to distribute your version of this file under
 * either the CDDL, the GPL Version 2 or to extend the choice of license to
 * its licensees as provided above.  However, if you add GPL Version 2 code
 * and therefore, elected the GPL Version 2 license, then the option applies
 * only if the new code is made subject to such option by the copyright
 * holder.
 */
package com.sun.enterprise.tools.verifier.tests.ejb.businessmethod;

import com.sun.enterprise.tools.verifier.tests.ejb.EjbTest;
import java.lang.reflect.*;
import com.sun.enterprise.deployment.*;
import com.sun.enterprise.tools.verifier.*;
import com.sun.enterprise.tools.verifier.tests.ejb.*;
import com.sun.enterprise.tools.verifier.tests.*;
import java.util.*;

/**  
 * Enterprise Bean's business(...) methods name test.
 * Each enterprise Bean class must define zero or more business(...) methods. 
 * The method signatures must follow these rules: 
 * 
 * The methods in the remote/local interface should be present in the deployment 
 * descriptor
 *
 */
public class BusinessMethodMatchesWithDD extends EjbTest implements EjbCheck { 

    Result result = null;
    ComponentNameConstructor compName = null;

    /** 
     * Enterprise Bean's business(...) methods name test.
     * Each enterprise Bean class must define zero or more business(...) methods. 
     * The method signatures must follow these rules: 
     * 
     * The methods in the remote/local interface should be present in the 
     * deployment descriptor
     *
     * @param descriptor the Enterprise Java Bean deployment descriptor
     * @return <code>Result</code> the results for this assertion
     */
    public Result check(EjbDescriptor descriptor) {

	result = getInitializedResult();
	compName = getVerifierContext().getComponentNameConstructor();

	if ((descriptor instanceof EjbSessionDescriptor)  ||
	    (descriptor instanceof EjbEntityDescriptor)) {
         
        if(descriptor.getRemoteClassName() != null && 
                !"".equals(descriptor.getRemoteClassName())) 
            commonToBothInterfaces(descriptor.getRemoteClassName(),
                                   descriptor, 
                                   MethodDescriptor.EJB_REMOTE);
        
        if(descriptor.getLocalClassName() != null && 
                !"".equals(descriptor.getLocalClassName())) 
            commonToBothInterfaces(descriptor.getLocalClassName(),
                                   descriptor, 
                                   MethodDescriptor.EJB_LOCAL);
	}
        
    if(result.getStatus() != Result.FAILED) {
        addGoodDetails(result, compName);
        result.passed(smh.getLocalString
                        (getClass().getName() + ".passed",
                        "Business method(s) are valid."));
    }
    return result;
    }
    
    /** 
     * This method is responsible for the logic of the test. It is called for 
     * both local and remote interfaces.
     * @param intf for the Remote/Local interface of the Ejb. 
     * @param descriptor the Enterprise Java Bean deployment descriptor
     * This parameter may be optional depending on the test 
     * @param methodIntf for the interface type
     */

    private void commonToBothInterfaces(String intf, 
                                        EjbDescriptor descriptor, 
                                        String methodIntf) {
        try {
            Class intfClass = Class.forName(intf, false, getVerifierContext().getClassLoader());
            
            boolean found = false;
            Set allMethods = new HashSet();
            
            for (Iterator e = 
                    descriptor.getPermissionedMethodsByPermission().values().iterator();e.hasNext();) {
                Set methodDescriptors = (Set) e.next();
                if (methodDescriptors != null)
                    allMethods.addAll(methodDescriptors);
            }
            for (Enumeration e = 
                    descriptor.getMethodContainerTransactions().keys();e.hasMoreElements();) {
                allMethods.add(e.nextElement());
            }
            
            for (Method remoteMethod : intfClass.getMethods()) {
                found = false;
                
                // we don't test the EJB methods
                if (remoteMethod.getDeclaringClass().getName().equals("javax.ejb.EJBObject")) 
                    continue;
                if (!remoteMethod.getName().startsWith("ejb") &&
                        !remoteMethod.getName().equals("class$") &&
                        !remoteMethod.getName().equals("setSessionContext")) {
                    
                    Iterator methods = allMethods.iterator();
                    while (methods.hasNext()) {
                        MethodDescriptor methodDescriptor = (MethodDescriptor)methods.next();
                        
                        if (methodDescriptor.getName().equals(remoteMethod.getName())) {
                            if (MethodUtils.stringArrayEquals(methodDescriptor.getParameterClassNames(), 
                                    (new MethodDescriptor(remoteMethod,methodIntf)).getParameterClassNames())) {
                                found = true;
                                break;
                            }
                        }
                    }
                }
                if (!found) {
                    addErrorDetails(result, compName);
                    result.failed(smh.getLocalString
                                    (getClass().getName() + ".failed",
                                    "Error: Business method [ {0} ] is not defined " +
                                    "in the deployment descriptor.",
                                    new Object[] {remoteMethod.getName()}));
                } 
            }
            
        } catch (Exception e) {
            Verifier.debug(e);
            addErrorDetails(result, compName);
            result.failed(smh.getLocalString
                            (getClass().getName() + ".failedException",
                            "Error: Component interface [ {0} ] does not exist " +
                            "or is not loadable within bean [ {1} ].",
                            new Object[] {intf,descriptor.getName()}));
        }  
    }
}
