/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 * 
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 * 
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common Development
 * and Distribution License("CDDL") (collectively, the "License").  You
 * may not use this file except in compliance with the License. You can obtain
 * a copy of the License at https://glassfish.dev.java.net/public/CDDL+GPL.html
 * or glassfish/bootstrap/legal/LICENSE.txt.  See the License for the specific
 * language governing permissions and limitations under the License.
 * 
 * When distributing the software, include this License Header Notice in each
 * file and include the License file at glassfish/bootstrap/legal/LICENSE.txt.
 * Sun designates this particular file as subject to the "Classpath" exception
 * as provided by Sun in the GPL Version 2 section of the License file that
 * accompanied this code.  If applicable, add the following below the License
 * Header, with the fields enclosed by brackets [] replaced by your own
 * identifying information: "Portions Copyrighted [year]
 * [name of copyright owner]"
 * 
 * Contributor(s):
 * 
 * If you wish your version of this file to be governed by only the CDDL or
 * only the GPL Version 2, indicate your decision by adding "[Contributor]
 * elects to include this software in this distribution under the [CDDL or GPL
 * Version 2] license."  If you don't indicate a single choice of license, a
 * recipient has the option to distribute your version of this file under
 * either the CDDL, the GPL Version 2 or to extend the choice of license to
 * its licensees as provided above.  However, if you add GPL Version 2 code
 * and therefore, elected the GPL Version 2 license, then the option applies
 * only if the new code is made subject to such option by the copyright
 * holder.
 */
package com.sun.enterprise.registration;

import java.util.logging.ConsoleHandler;
import java.util.logging.Handler;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 * Provides access to a logger for registration.  Uses the logger name
 * com.sun.enterprise.registration.
 *<p>
 * The system property com.sun.enterprise.registration.logging.level can be set to
 * any legal logging level value and that will be used as the initial level
 * for the logger.
 * 
 */
public class RegistrationLogger {
    
    private static Logger logger = initLogger();

    private RegistrationLogger() {}

    private static Logger initLogger() {
        Logger l = Logger.getLogger(RegistrationLogger.class.getPackage().getName());
        String levelText = System.getProperty(RegistrationLogger.class.getPackage().getName() + ".logging.level", "WARNING");
        Level level = Level.parse(levelText);
        l.setLevel(level);
        
        Handler h = new ConsoleHandler();
        h.setLevel(level);
        l.addHandler(h);
        
        return l;
    }

    /**
     * Returns the logger to be used for logging registration-related
     * messages.
     */
    public static Logger getLogger() {
        return logger;
    }

    /**
     * Overrides the default logger with one of the caller's choosing.
     * @param logger the Logger to use in place of the default one
     */
    public static void setLogger(Logger logger) {
        RegistrationLogger.logger = logger;
    }
}
