/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 * 
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 * 
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common Development
 * and Distribution License("CDDL") (collectively, the "License").  You
 * may not use this file except in compliance with the License. You can obtain
 * a copy of the License at https://glassfish.dev.java.net/public/CDDL+GPL.html
 * or glassfish/bootstrap/legal/LICENSE.txt.  See the License for the specific
 * language governing permissions and limitations under the License.
 * 
 * When distributing the software, include this License Header Notice in each
 * file and include the License file at glassfish/bootstrap/legal/LICENSE.txt.
 * Sun designates this particular file as subject to the "Classpath" exception
 * as provided by Sun in the GPL Version 2 section of the License file that
 * accompanied this code.  If applicable, add the following below the License
 * Header, with the fields enclosed by brackets [] replaced by your own
 * identifying information: "Portions Copyrighted [year]
 * [name of copyright owner]"
 * 
 * Contributor(s):
 * 
 * If you wish your version of this file to be governed by only the CDDL or
 * only the GPL Version 2, indicate your decision by adding "[Contributor]
 * elects to include this software in this distribution under the [CDDL or GPL
 * Version 2] license."  If you don't indicate a single choice of license, a
 * recipient has the option to distribute your version of this file under
 * either the CDDL, the GPL Version 2 or to extend the choice of license to
 * its licensees as provided above.  However, if you add GPL Version 2 code
 * and therefore, elected the GPL Version 2 license, then the option applies
 * only if the new code is made subject to such option by the copyright
 * holder.
 */

package com.sun.appserv.web.cache.filter;

import java.util.HashMap;
import java.util.ArrayList;
import java.util.Locale;

/** HttpCacheEntry 
 *  Each entry holds cached (HTTP) response:
 *  a) response bytes b) response headers c) expiryTime 
 *  d) parameterEncoding used e) entryKey this entry represents, 
 *  to match the entry within the hash bucket. 
 *
 *  XXX: should implement methods to enable serialization of cached response?
 */
public class HttpCacheEntry {

    public static final int VALUE_NOT_SET = -1;

    int statusCode;
    String statusMessage;

    HashMap responseHeaders;
    HashMap dateHeaders;
    ArrayList cookies;
    String contentType;
    Locale locale;

    int contentLength;

    // XXX: other cacheable response info 
    byte[] bytes; 

    int timeout;
    volatile long expireTime = 0;

    /**
     * set the real expire time
     * @param expireTime in milli seconds
     */
    public void setExpireTime(long expireTime) {
        this.expireTime = expireTime;
    }

    /**
     * compute when this entry to be expired based on timeout relative to 
     * current time.
     * @param timeout in seconds
     */
    public void computeExpireTime(int timeout) {
        this.timeout = timeout;

        // timeout is relative to current time
        this.expireTime = (timeout == -1) ? timeout :
                          System.currentTimeMillis() + (timeout * 1000);
    }

    /**
     * is this response still valid?
     */
    public boolean isValid() {
        return (expireTime > System.currentTimeMillis() || expireTime == -1);
    }

    /** 
     * clear the contents
     */
    public void clear() {
        bytes = null;
        responseHeaders = null;
        cookies = null;
    }

    /**
     * get the size 
     * @return size of this entry in bytes
     * Note: this is only approximate
     */
    public int getSize() {
        int size = 0;
        if (bytes != null) {
            size = bytes.length;
        }

        // size of response bytes plus headers (each approx 20 chars or 40 bytes)
        return (size + (40 * responseHeaders.size()) );
    }
}
